import { Http, IDictionary, IEventParam, Loader, Metadata, Singleton } from '@zeedhi/core';
import { ITreeNode, ITreeNodeModel, Tree } from '@zeedhi/common';
import { PmLibDialogService, PmLibHttp, PmLibUtil } from '@zeedhi/pm-lib';
import { PerfilDeAcessoController } from '@/controllers/PerfilDeAcesso/PerfilDeAcessoController';
import { TekLibLocalStorage } from '@zeedhi/tek-lib/index';
import { NodeType } from '@/services/PerfilDeAcesso/NodeType';
import { NodeTypeService } from '@/services/PerfilDeAcesso/NodeTypeService';

@Singleton
export class TabLabelsTreeController {
	public perfilDeAcessoControler: PerfilDeAcessoController;

	constructor() {
		this.perfilDeAcessoControler = Loader.getInstance('PerfilDeAcessoController');
	}

	public originalNodes: any = [];

	private labelsTreeObject?: Tree;

	private labelsToBeSaved: any = [];

	private rowsToSaveOldAcl: any = [];

	get labelsTreeComponent(): Tree {
		if (!this.labelsTreeObject) {
			this.labelsTreeObject = Metadata.getInstance<Tree>('labelsTree');
		}
		return this.labelsTreeObject;
	}

	get isAdmin(): boolean {
		return this.perfilDeAcessoControler.currentPerfilDeAcesso?.IDPERFILACESSOADM === 'S';
	}

	public disableCheckbox({ node }: { node: ITreeNode<IDictionary> }) {
		return node.data?.disabled;
	}

	public async onLoadLabelsTree(versionSave = false) {
		const oldMajorVersion = TekLibLocalStorage.getItem('oldMajorVersion');
		const params = {
			params: {
				nrperfilacesso: this.perfilDeAcessoControler.currentPerfilDeAcesso.NRPERFILACESSO,
				oldMajorVersion,
				versionSave,
				isNext: TekLibLocalStorage.getItem('IS_NEXT') !== false,
				productId: TekLibLocalStorage.getItem('PRODUCT_ID'),
				nrpperfilacesso: this.perfilDeAcessoControler.currentPerfilDeAcesso.NRPERFILACESSO,
			},
		};
		const { data: { data } } = await PmLibHttp.$get('/perfilacessoLabel', params);
		this.originalNodes = PmLibUtil.clone(data);
		this.labelsTreeComponent.nodes = PmLibUtil.clone(this.originalNodes);
		/**
		 * Método: checkAndFixNodesVisibility
		 *
		 * Responsável por fazer com que elementos pais
		 * que tenham filhos checkados e não estão salvos
		 * nos banco de dados sejam checados tambem
		 */
		this.checkAndFixNodesVisibility(this.labelsTreeComponent.nodes);

		if (this.isAdmin) {
			this.setAllCheckedInNodes(this.labelsTreeComponent.nodes);
		}
	}

	private checkAndFixNodesVisibility(nodes: Array<ITreeNodeModel<IDictionary>>) {
		nodes.forEach((node, index, arrayNodes) => {
			if (node.data?.checked && node.data?.parent) {
				this.setParentNodeCheckedValue(this.labelsTreeComponent.nodes, node.data.parent);
			}

			if (node.children?.length) {
				this.checkAndFixNodesVisibility(node.children);
			}

			if (node.data?.type === NodeType.grouped && !node.children?.length) {
				arrayNodes.splice(index, 1);
			}
		});
	}

	private setParentNodeCheckedValue(nodes: Array<ITreeNodeModel<any>>, parent: string) {
		nodes.some((node) => {
			const conditionIsNotGrouped = node.data?.type !== NodeType.grouped && node.data?.label === parent;
			const conditionIsGrouped = node.data?.type === NodeType.grouped && node.title === parent;
			if (conditionIsNotGrouped) {
				node.data.checked = true;

				if (node.data?.parent) {
					this.setParentNodeCheckedValue(this.labelsTreeComponent.nodes, node.data.parent);
				}
			}

			if (conditionIsGrouped) {
				node.data.checked = true;
			}

			if (node.children?.length) {
				this.setParentNodeCheckedValue(node.children, parent);
			}

			return conditionIsNotGrouped || conditionIsGrouped;
		});
	}

	private setAllCheckedInNodes(nodes: Array<ITreeNodeModel<IDictionary>>) {
		nodes.forEach((node) => {
			node.data!.checked = true;
			node.data!.disabled = true;
			if (node.children?.length) {
				this.setAllCheckedInNodes(node.children);
			}
		});
	}

	public getIconName(node: ITreeNodeModel<any>) {
		if (node.data.checked) {
			return 'mdi-eye';
		}
		return 'mdi-eye-off';
	}

	public getIconColor(node: ITreeNodeModel<any>) {
		if (node.data.checked) {
			return '#2d68c3';
		}
		return '#ccc';
	}

	public getNodeTitle(node: ITreeNodeModel<any>) {
		return node.title;
	}

	public getNodeTypeLabel(node: ITreeNodeModel<any>) {
		return NodeTypeService.getNodeTypeLabel(node.data.type);
	}

	public getNodeTypeColor(node: ITreeNodeModel<any>) {
		return NodeTypeService.getNodeTypeColor(node.data.type);
	}

	public getNodeIsNew(node: ITreeNodeModel<any>) {
		return node.data.new;
	}

	/** Actions Tree */

	public async clickAclLabelsTreeButtonReload() {
		this.onLoadLabelsTree();
	}

	public clickAclLabelsTreeButtonExpand() {
		this.setIsExpandedAllNodes(this.labelsTreeComponent.nodes, true);
	}

	public clickAclLabelsTreeButtonCollapse() {
		this.setIsExpandedAllNodes(this.labelsTreeComponent.nodes, false);
	}

	private setIsExpandedAllNodes(nodes: Array<ITreeNodeModel<any>>, value: boolean) {
		nodes.forEach((node) => {
			node.isExpanded = value;
			if (node.children?.length) {
				this.setIsExpandedAllNodes(node.children, value);
			}
		});
	}

	/** Events Tree */

	public async onActiveTab() {
		this.onLoadLabelsTree(true);
	}

	public async onNodeCheck({ node }: IEventParam<Tree>) {
		if (!this.isAdmin) {
			node.data.checked = !node.data.checked;
			const nrperfilacesso = this.perfilDeAcessoControler.currentPerfilDeAcesso.NRPERFILACESSO;
			const newVisibilityValue = node.data.checked;

			this.addLabelsToBeSavedNotIfTypeGrouped(node);
			/** Old ACL */
			this.addRowsToBeSavedOldAcl(node, newVisibilityValue);

			this.addLabelsChildrenNodesToSave(node, newVisibilityValue);

			if (newVisibilityValue) {
				this.addLabelsParentNodeCheckedToSave(node, newVisibilityValue);
				if (!TekLibLocalStorage.getItem('IS_NEXT')) {
					this.saveZhuaclElementsRules(this.rowsToSaveOldAcl);
				}
				await this.savePerfilacessoLabels(this.labelsToBeSaved, nrperfilacesso);
			} else {
				if (node.data.type !== NodeType.rule) {
					this.addLabelsParentNodeUncheckedToSave(node, newVisibilityValue);
				}
				if (!TekLibLocalStorage.getItem('IS_NEXT')) {
					this.saveZhuaclElementsRules(this.rowsToSaveOldAcl);
				}
				await this.deletePerfilacessoLabels(this.labelsToBeSaved, nrperfilacesso);
			}
			this.labelsToBeSaved = [];

			/** Old ACL */
			this.rowsToSaveOldAcl = [];
			this.onLoadLabelsTree();
		}
	}

	private addLabelsChildrenNodesToSave(node: ITreeNodeModel<any>, newVisibilityValue: boolean) {
		node.children?.forEach((child: ITreeNodeModel<any>) => {
			this.addLabelsToBeSavedNotIfTypeGrouped(child);
			/** Old ACL */
			this.addRowsToBeSavedOldAcl(child, newVisibilityValue);
			this.addLabelsChildrenNodesToSave(child, newVisibilityValue);
		});
	}

	private addLabelsParentNodeCheckedToSave(node: ITreeNodeModel<any>, newVisibilityValue: boolean) {
		try {
			const parent = this.labelsTreeComponent.getParentNode(node);
			parent.data.checked = newVisibilityValue;
			this.addLabelsToBeSavedNotIfTypeGrouped(parent);
			/** Old ACL */
			this.addRowsToBeSavedOldAcl(parent, newVisibilityValue);
			this.addLabelsParentNodeCheckedToSave(parent, newVisibilityValue);
		} catch (e) {} // eslint-disable-line no-empty
	}

	public addLabelsParentNodeUncheckedToSave(node: ITreeNodeModel<any>, newVisibilityValue: boolean) {
		try {
			const parent = this.labelsTreeComponent.getParentNode(node);
			const hasChildVisible = this.hasChildVisible(parent.children);
			if (!hasChildVisible) {
				parent.data.checked = false;
				this.addLabelsToBeSavedNotIfTypeGrouped(parent);
				/** Old ACL */
				this.addRowsToBeSavedOldAcl(parent, newVisibilityValue);
				this.addLabelsParentNodeUncheckedToSave(parent, newVisibilityValue);
			}
		} catch (e) {} // eslint-disable-line no-empty
	}

	private hasChildVisible(nodeChildren: Array<ITreeNodeModel<any>>): boolean {
		return nodeChildren.some((node) => node.data.checked);
	}

	private addLabelsToBeSavedNotIfTypeGrouped(node: ITreeNodeModel<any>) {
		if (node && node.data.type !== NodeType.grouped) {
			this.labelsToBeSaved.push(node.data.label);
		}
	}

	private async savePerfilacessoLabels(labels: Array<string>, nrperfilacesso: string | Number) {
		const data: any = { labels, nrperfilacesso };
		await PmLibHttp.$post('/perfilacessoLabel', data);
	}

	private async deletePerfilacessoLabels(labels: Array<string>, nrperfilacesso: string | Number) {
		const data: any = { labels, nrperfilacesso };
		await PmLibHttp.$delete('/perfilacessoLabel', { data });
	}

	/** Old ACL Section */

	private addRowsToBeSavedOldAcl(node: ITreeNodeModel<any>, newVisibilityValue: boolean) {
		const row: any = this.createRow(node, newVisibilityValue);

		if (this.hasContainerIdInNode(node)) {
			row.CONTAINER_ID = node.data.container.id;
			row.CONTAINER_NAME = node.data.container.name;
			row.CONTAINER_LABEL = node.data.container.label;
		}

		if (row.NAME) {
			this.rowsToSaveOldAcl.push(row);
		}
	}

	private createRow(node: ITreeNodeModel<any>, newVisibilityValue: boolean): object {
		return {
			GROUP_ID: this.perfilDeAcessoControler.currentPerfilDeAcesso.NRPERFILACESSO,
			LABEL: node.title,
			NAME: node.data.name,
			ORGANIZATION_ID: this.perfilDeAcessoControler.currentNrorg,
			PROJECT_ID: TekLibLocalStorage.getItem('PRODUCT_ID'),
			RULE: newVisibilityValue ? 'A' : 'I',
			TYPE: 'M',
			UNIQUE_ID: node.data.label,
			USER_ID: this.perfilDeAcessoControler.currentCdoperador,
			changed: true,
		};
	}

	private hasContainerIdInNode(node: ITreeNodeModel<any>) {
		return (node.data.container && node.data.container.id);
	}

	private async saveZhuaclElementsRules(dataset: Array<object>) {
		const config = { dataset, requestType: 'DataSet' };

		try {
			await Http.post('/acl_zhu_acl_element_rules/save', config);
		} catch (e) {
			PmLibDialogService.showErrorBackend(e);
		}
	}
}
